<?php
// $Id: moodle.cckfield.inc,v 1.1 2009/02/19 18:09:26 chris Exp $

/**
 * @file
 * Optional CCK extension provides read-only fields which display Moodle data.
 */

require_once('moodle.rest.inc');

// Public constants
define('MOODLE_UPCOMING_COURSES', 'moodle_upcoming_courses');
define('MOODLE_USER_COURSES', 'moodle_user_courses');

/**
 * Implementation of hook_theme().
 */
function moodle_theme() {
  return array(
    'moodle_field'                => array('arguments' => array('element' => NULL)),
    'moodle_field_format_default' => array('arguments' => array('element' => NULL)),
    'moodle_field_format_plain'   => array('arguments' => array('element' => NULL)),
    'moodle_field_format_teaser'  => array('arguments' => array('element' => NULL)),
    'moodle_field_format_full'    => array('arguments' => array('element' => NULL)),
  );
}


/**
 * Implementation of CCK hook_field_info().
 */
function moodle_field_info() {
  return array(
    MOODLE_UPCOMING_COURSES => array(
      'label' => t('Moodle upcoming courses'),
      'description' => t('A list of upcoming Moodle courses.'),
    ),
    MOODLE_USER_COURSES => array(
      'label' => t('Moodle user courses'),
      'description' => t('A list of Moodle courses for a specific person.'),
    ),
  );
}


/**
 * Implementation of CCK hook_field_settings().
 */
function moodle_field_settings($op, $field) {
  switch ($op) {
    case 'form' :
      $form = array();

      $form['multiple'] = $form['required'] = array('#type' => 'value');

      $form['result_limit'] = array(
        '#type' => 'textfield',
        '#title' => t('Number of courses to display'),
        '#description' => t('Moodle courses to display per page.  Must be more  than 0.'),
        '#default_value' => is_numeric($field['result_limit']) ? $field['result_limit'] : 5,
        '#required' => TRUE,
      );

      $form['moodle_header'] = array(
        '#tree' => TRUE,
        '#title' => t('Header Text'),
        '#type' => 'fieldset',
        '#description' => t('Optionally include some text to appear above the   list and below the label (if the label is enabled).'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
      );
      $form['moodle_header']['body'] = array(
        '#type' => 'textarea',
        '#default_value' => isset($field['moodle_header']['body']) ? $field['moodle_header']['body'] : '',
      );
      $form['moodle_header']['format'] = filter_form($field['moodle_header']['format'], NULL, array('moodle_header', 'format'));

      return $form;

    case 'save' :
      return array('result_limit', 'moodle_header');

    case 'database columns':
      $columns['value'] = array('type' => 'text', 'size' => 'big', 'not null'   => FALSE, 'sortable' => FALSE);
      return $columns;

    case 'views data':
      $allowed_values = content_allowed_values($field);
      if (count($allowed_values)) {
        $data = content_views_field_views_data($field);
        $db_info = content_database_info($field);
        $table_alias = content_views_tablename($field);

        // Swap the filter handler to the 'in' operator.
        $data[$table_alias][$field['field_name'] .                              '_value']['filter']['handler'] = 'content_handler_filter_many_to_one';
        $data[$table_alias][$field['field_name'] .                              '_value']['argument']['handler'] = 'content_handler_argument_many_to_one';
        return $data;
      }
  } // end switch
}


/**
 * Implementation of CCK hook_field().
 * Note: argument 4, $items is often called $node_field in other CCK modules
 */
function moodle_field($op, &$node, $field, &$items, $teaser, $page) {
  global $user;     // yuck, fixme?
  switch ($op) {
    case 'load':
      // get data from Moodle if not stored (up to date -- todo) in database
      if ($field['store']) {
        // yes, in database, just return
        break;
      }

      // get the data from Moodle based upon type of data desired
      switch ($field['type']) {
      case MOODLE_UPCOMING_COURSES:
        $courses = _moodle_get_courses();
        break;

      case MOODLE_USER_COURSES:
        $courses = _moodle_get_usercourses($user->name);
        break;

      default:
        watchdog('moodle', 'The CCK field type %type is not recognized.', array('%type' => $field['type'], WATCHDOG_ERROR));
      }

      if (isset($courses['error']) || (count($courses) < 1)) {
        $items = array(0 => array(
            'urlpath' => '',
            'title' => '',
            'summary' => 'No courses found',
            'startdate' => ''
          )
        );
      }
      else {
        // adjust size to requested limit
        $items = array_slice($courses, 0, $field['result_limit']);
      }

      return array(
        $field['field_name'] => array(
          'items' => $items,
          'header' => $field['moodle_header'],
          '#theme' => 'moodle_field',
        )
      );

    case 'sanitize' :
      if (!empty($items['header'])) {
        $items['header']['safe'] = check_markup($items['header']['body'], $items['header']['format'], FALSE);
      }
      break;

    case 'insert':
    case 'update':
      $items = array_slice(_moodle_get_courses(), 0, $field['result_limit']);
      break;
  }
}


/**
 * Implementation of CCK hook_field_formatter_info().
 */
function moodle_field_formatter_info() {
  return array(
    'default' => array('multiple values' => CONTENT_HANDLE_MODULE, 'field types' => array(MOODLE_UPCOMING_COURSES, MOODLE_USER_COURSES), 'label' => t('Default course view')),
    'plain'   => array('multiple values' => CONTENT_HANDLE_MODULE, 'field types' => array(MOODLE_UPCOMING_COURSES, MOODLE_USER_COURSES), 'label' => t('Plaintext course title')),
    'teaser'  => array('multiple values' => CONTENT_HANDLE_MODULE, 'field types' => array(MOODLE_UPCOMING_COURSES, MOODLE_USER_COURSES), 'label' => t('Teaser view')),
    'full'    => array('multiple values' => CONTENT_HANDLE_MODULE, 'field types' => array(MOODLE_UPCOMING_COURSES, MOODLE_USER_COURSES), 'label' => t('Full View')),
  );
}


/**
 * Field theming functions
 */
function theme_moodle_field_format_default($element) {
  return _moodle_field_format_item($element); 
}

function theme_moodle_field_format_plain($element) {
  return check_plain($element['title']);
}

function theme_moodle_field_format_teaser($element) {
  return _moodle_field_format_item($element);   // todo
}

function theme_moodle_field_format_full($element) {
  return _moodle_field_format_item($element);   // todo
}

function _moodle_field_format_item($element) {
  $title = $element['title'];
  $url = '#';
  if (!empty($element['urlpath'])) {
    $url = _moodle_get_url() . $element['urlpath'];
  }
  $summary = $element['summary'];
  $startdate = '';
  if (isset($element['startdate']) && !empty($element['startdate'])) {
    // $startdate = ' on '. $element['startdate'];
  }
  return "<li>" . _moodle_link($url, $title) . " $summary $startdate </li>";
}

/**
 * Implementation of CCK hook_content_is_empty().  REQUIRED!
 * Wtf this is needed here but not in relevant content module is beyond me.
 */
function moodle_content_is_empty($item, $field) {
  return FALSE;     // our field is never considered empty
}


/**
 * Implementation of CCK hook_widget_info().
 */
function moodle_widget_info() {
  return array(
    'moodle_courses' => array(
      'label' => t('Moodle course display (Read Only)'),
      'field types' => array(MOODLE_UPCOMING_COURSES, MOODLE_USER_COURSES),
      'multiple values' => CONTENT_HANDLE_MODULE,
      'callbacks' => array('default value' => CONTENT_CALLBACK_NONE),
    ),
  );
}


/**
 * Implementation of CCK hook_widget().
 */
function moodle_widget(&$form, &$form_state, $field, $items, $delta = 0) {
 return array('#type' => $field['widget']['type'], '#default_value' => 'WIDGET');  /* DEBUG */ // todo

  return array();
}


/**
 * Theme function for outputing a moodle field.
 * Called once per field.
 */
function theme_moodle_field($element) {
  $items_output = '';
  $format = $element['#formatter'];
  $count = 1;
  foreach ($element['items']['#item'] as $delta => $item) {
    $items_output .= '<div class="field-item '. ($count % 2 ? 'odd' : 'even') .'">'. theme('moodle_field_format_'. $format, $item) .'</div>';
    $count++;
  }

  $output = '';
  if (!empty($items_output)) {
    $output .= '<div class="field field-type-moodle field-'. check_plain(strtr($element['#field_name'], '_', '-')) .'">';
    if (!empty($element['header']['#item']['safe'])) {
      $output .= '<div class="field-header description">'. $element['header']['#item']['safe'] .'</div>';
    }
    $output .= '<div class="field-items">'. $items_output .'</div>';
    $output .= '</div>';
  }
  return $output;
}

/**
 * DEBUG!
 */
function cj_print_stack() {
    $output = "<pre>";
    foreach (debug_backtrace() as $row) {
        if ($last != $row['file']) {
            $output .= "File: {$row['file']}\n";
        }
        $last = $row['file'];
        $output .= sprintf("%s: %s\n", $row['line'], $row['function']);
    }
    $output .= "</pre>";
    drupal_set_message($output);
}

