<?php
// $Id: moodle.rest.inc,v 1.1 2009/02/19 18:09:27 chris Exp $

/**
 * @file
 * Defines the client connection to the RESTful web service provided
 * by Moodle.
 */

/**
 * Get courses for a date range from the Moodle web service.
 *
 * @param $start
 *  Starting date for courses of interest in YYYYMMDD format.
 * @param $end
 *  Ending date for courses of interest in YYYYMMDD format.
 * 
 * @return -  array of course objects on success, FALSE on error
 */
function _moodle_get_courses($start = NULL, $end = NULL) {
  if (($moodle_url = _moodle_get_url()) === FALSE) return FALSE;

  if (is_null($start)) {
    $start = date('Ymd');
  }
  if (is_null($end)) {
    $end = date('Ymd', time() + (86400 * 30));
  }

  $url = $moodle_url . "spawar/rest.php/course/upcoming";

  return _moodle_get_resource($url);
}


/**
 * Get courses for a user from the Moodle web service.
 * 
 * @return -  array of course objects on success, FALSE on error
 */
function _moodle_get_usercourses($username, $status = 'enrolled') {
  // todo: should add some username validation code here before passing it
  $username = urlencode($username);

  if (($moodle_url = _moodle_get_url()) === FALSE) return FALSE;

  // $url = $moodle_url . 'spawar/rest.php/course/user/' . $username;
  $url = sprintf('%s/spawar/rest.php/course/user/%s/%s', $moodle_url, $username, $status);

  return _moodle_get_resource($url);
}

/**
 * get a Moodle resource.
 */
function _moodle_get_resource($url) {
  $reply = drupal_http_request($url, $headers = array(), $method = 'GET', $data = NULL, $retry = 3);
  if (isset($reply->error)) {
    drupal_set_message("drupal_hhtp_request($url) failed");  //  /* DEBUG */
    watchdog('moodle', 'Moodle web service returned an error.', array(), WATCHDOG_ERROR);
  }

  $resource = json_decode($reply->data, TRUE);
  return $resource;
}

// --- END OF FILE ---
